unit BmpLoadW;

interface

uses
  Classes, Controls, Dialogs, ExtCtrls, Forms, StdCtrls, SysUtils, WinTypes,
  Graphics;

const
  NextProcsCount = 4;

procedure ColorToRGB(C: TColor; var R, G, B: Longint);
function ColorDistance(Color1, Color2: TColor): Longint;
function ColorToMuseColorIndex(AColor: TColor): Integer;

type
  EColorListError = class(Exception);

  TColorListNode = record
    Color: TColor;
    MuseColorIndex: Integer;
  end;

  TColorList = class
  public
    Count: Integer;
    Items: array[0..255] of TColorListNode;
    procedure Add(AColor: TColor);
    procedure Clear;
    function DisplayString(AIndex: Integer): String;
    procedure Sort;
  end;

type
  TNextProc = procedure of object;

  TLoadBitmapWizard = class(TForm)
    BasePanel: TPanel;
     FileLoadNotebook: TNotebook;
      {Page FileName}
      FileNameLabel: TLabel;
       FileNameEdit: TEdit;
        ReferenceButton: TButton;
      PreviewScrollBox: TScrollBox;
       PreviewImage: TImage;
      {Page Stretch}
      StretchMessageLabel: TLabel;
      CutoffRadioButton: TRadioButton;
       CutoffScrollBox: TScrollBox;
        CutoffImage: TImage;
      ReduceScrollBox: TScrollBox;
       ReduceImage: TImage;
        ReduceRadioButton: TRadioButton;
      ReduceVScrollBox: TScrollBox;
       ReduceVImage: TImage;
        ReduceVRadioButton: TRadioButton;
      {Page ColorConvert}
      ColorConvertLabel: TLabel;
      ColorConvertListBox: TListBox;
      ColorBoxGroupBox: TGroupBox;
       LabelW: TLabel;
        ColorPanelW: TPanel;
       LabelA: TLabel;
        ColorPanelA: TPanel;
       LabelB: TLabel;
        ColorPanelB: TPanel;
       LabelC: TLabel;
        ColorPanelC: TPanel;
       LabelD: TLabel;
        ColorPanelD: TPanel;
       LabelE: TLabel;
        ColorPanelE: TPanel;
       LabelF: TLabel;
        ColorPanelF: TPanel;
       LabelG: TLabel;
        ColorPanelG: TPanel;
       LabelH: TLabel;
        ColorPanelH: TPanel;
       LabelI: TLabel;
        ColorPanelI: TPanel;
       LabelJ: TLabel;
        ColorPanelJ: TPanel;
       LabelK: TLabel;
        ColorPanelK: TPanel;
       LabelL: TLabel;
        ColorPanelL: TPanel;
       LabelM: TLabel;
        ColorPanelM: TPanel;
       LabelN: TLabel;
        ColorPanelN: TPanel;
       LabelO: TLabel;
        ColorPanelO: TPanel;
       LabelZ: TLabel;
        ColorPanelZ: TPanel;
      ColorConvertScrollBox: TScrollBox;
       ColorConvertImage: TImage;
    BackButton: TButton;
    NextButton: TButton;
    CancelButtonButton: TButton;
    BitmapOpenDialog: TOpenDialog;
    StartTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FileNameEditChange(Sender: TObject);
    procedure ReferenceButtonClick(Sender: TObject);
    procedure ColorConvertListBoxDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure ColorConvertListBoxDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure ColorConvertListBoxDrawItem(Control: TWinControl;
      Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure BackButtonClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure StartTimerTimer(Sender: TObject);
  private
    FFileName: TFileName;
  protected
    ColorList: TColorList;
    NextPhase: Integer;
    NextProcs: array[1..NextProcsCount] of TNextProc;
    SourceBitmap: TBitmap;
    StayFileNamePage: Boolean;
    StoredFileName: TFileName;
    procedure DrawColorConvertImage;
    procedure NextProc1;
    procedure NextProc2;
    procedure NextProc3;
    procedure NextProc4;
    procedure SetFileName(AFileName: TFileName);
  public
    Modified: Boolean;
    property FileName: TFileName read FFileName write SetFileName;
    function Execute(AFileName: TFileName): Boolean;
    function NotesBitmap: TBitmap;
  end;

var
  LoadBitmapWizard: TLoadBitmapWizard;

implementation

{$R *.DFM}

uses
  MsPaintF;

procedure ColorToRGB(C: TColor; var R, G, B: Longint);
begin
 R := C and $0000FF;
 G := (C and $00FF00) div $100;
 B := (C and $FF0000) div $10000;
end;

function ColorDistance(Color1, Color2: TColor): Longint;
var
 R1, G1, B1: Longint;
 R2, G2, B2: Longint;
begin
 ColorToRGB(Color1, R1, G1, B1);
 ColorToRGB(Color2, R2, G2, B2);
 Result := (R2 - R1) * (R2 - R1)
          + (G2 - G1) * (G2 - G1)
           + (B2 - B1) * (B2 - B1)
end;

function ColorToMuseColorIndex(AColor: TColor): Integer;
var
 i: Integer;
 Distance, D: Longint;
begin
 Result := 0;
 Distance := $7FFFFFFF;
 for i := 0 to ColorsCount do
  begin
   D := ColorDistance(AColor, MuseColors[i]);
   if D < Distance then
    begin
     Distance := D;
     Result := i;
    end;
  end;
end;

procedure TColorList.Add(AColor: TColor);
var
 i: Integer;
begin
 for i := 0 to Count - 1 do
  if Items[i].Color = AColor then   Exit;
 if Count = 256 then
  raise EColorListError.Create('');
 Items[Count].Color := AColor;
 Items[Count].MuseColorIndex := ColorToMuseColorIndex(AColor);
 if not(MuseColors[Items[Count].MuseColorIndex] = AColor) then
  Count := Count + 1;
end;

procedure TColorList.Clear;
begin
 Count := 0;
end;

function TColorList.DisplayString(AIndex: Integer): String;
var
 R, G, B: Longint;
begin
 ColorToRGB(Items[AIndex].Color, R, G, B);
 Result := 'R:' + IntToHex(R, 2) +
            ' G:' + IntToHex(G, 2) +
             ' B:' + IntToHex(B, 2) +
              ' -> ' +
               MuseColorNames[Items[AIndex].MuseColorIndex];
end;

procedure TColorList.Sort;

  function CompareColorListNode(Node1, Node2: TColorListNode): Integer;
  var
   R1, G1, B1, R2, G2, B2: Longint;
  begin
   if Node1.MuseColorIndex < Node2.MuseColorIndex then
    begin
     Result := 1;
     Exit;
    end;
   if Node1.MuseColorIndex > Node2.MuseColorIndex then
    begin
     Result := -1;
     Exit;
    end;
   ColorToRGB(Node1.Color, R1, G1, B1);
   ColorToRGB(Node2.Color, R2, G2, B2);
   if R1 < R2 then
    begin
     Result := 1;
     Exit;
    end;
   if R1 > R2 then
    begin
     Result := -1;
     Exit;
    end;
   if G1 < G2 then
    begin
     Result := 1;
     Exit;
    end;
   if G1 > G2 then
    begin
     Result := -1;
     Exit;
    end;
   if B1 < B2 then
    begin
     Result := 1;
     Exit;
    end;
   if B1 > B2 then
    begin
     Result := -1;
     Exit;
    end;
   Result := 0;
  end;

var
 TempNode: TColorListNode;
 i, j, Index: Integer;
begin
 for i := 0 to Count - 1 do
  begin
   Index := i;
   for j := i to Count - 1 do
    if 0 < CompareColorListNode(Items[j], Items[Index]) then
     Index := j;
   TempNode := Items[i];
   Items[i] := Items[Index];
   Items[Index] := TempNode;
  end;
end;

procedure TLoadBitmapWizard.FormCreate(Sender: TObject);
begin
 NextProcs[1] := NextProc1;
 NextProcs[2] := NextProc2;
 NextProcs[3] := NextProc3;
 NextProcs[4] := NextProc4;
 ColorList := TColorList.Create;
 ColorConvertListBox.Canvas.Font := ColorConvertListBox.Font;
end;

procedure TLoadBitmapWizard.FormShow(Sender: TObject);
begin
 StartTimer.Enabled := True;
end;

procedure TLoadBitmapWizard.FormDestroy(Sender: TObject);
begin
 ColorList.Free;
end;

procedure TLoadBitmapWizard.StartTimerTimer(Sender: TObject);
begin
 StartTimer.Enabled := False;
 NextPhase := 1;
 NextProcs[NextPhase];
end;

procedure TLoadBitmapWizard.FileNameEditChange(Sender: TObject);
begin
 FileName := FileNameEdit.Text;
end;

procedure TLoadBitmapWizard.ReferenceButtonClick(Sender: TObject);
begin
 if BitmapOpenDialog.Execute then
  begin
   FileName := BitmapOpenDialog.FileName;
   BitmapOpenDialog.InitialDir := ExtractFilePath(FileName);
   BitmapOpenDialog.FileName := ExtractFileName(FileName);
  end;
 ActiveControl := FileNameEdit;
end;

procedure TLoadBitmapWizard.ColorConvertListBoxDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
 Index, TopIndex: Integer;
begin
 TopIndex := ColorConvertListBox.TopIndex;
 Index := (TopIndex * ColorConvertListBox.ItemHeight + Y)
            div ColorConvertListBox.ItemHeight;
 if ColorList.Count - 1 < Index then   Exit;
 ColorList.Items[Index].MuseColorIndex := TPanel(Source).Tag;
 ColorConvertListBox.Items[Index] := ColorList.DisplayString(Index);
 ColorConvertListBox.TopIndex := TopIndex;
 DrawColorConvertImage;
end;

procedure TLoadBitmapWizard.ColorConvertListBoxDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
 Accept := Source is TPanel;
end;

procedure TLoadBitmapWizard.ColorConvertListBoxDrawItem(
  Control: TWinControl; Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
begin
 with TListBox(Control).Canvas do
  begin
   Brush.Color := clBtnFace;
   Brush.Style := bsSolid;
   FillRect(Rect);
   Pen.Color := clBtnHighlight;
   Pen.Mode  := pmCopy;
   Pen.Style := psSolid;
   Pen.Width := 1;
   MoveTo(Rect.Left , Rect.Bottom - 1);
   LineTo(Rect.Left , Rect.Top);
   LineTo(Rect.Right - 1, Rect.Top);
   Pen.Color := clBtnShadow;
   LineTo(Rect.Right - 1, Rect.Bottom - 1);
   LineTo(Rect.Left , Rect.Bottom - 1);
   Pen.Color := clBlack;
   Brush.Color := ColorList.Items[Index].Color;
   RectAngle(Rect.Left + 8, Rect.Top + 1, Rect.Left + 48, Rect.Top + 15);
   Brush.Color := clBtnFace;
   TextOut(Rect.Left + 56, Rect.Top + 1, ColorList.DisplayString(Index));
   Brush.Color := MuseColors[ColorList.Items[Index].MuseColorIndex];
   RectAngle(Rect.Left + 200, Rect.Top + 1, Rect.Left + 240, Rect.Top + 15);
  end;
end;

procedure TLoadBitmapWizard.BackButtonClick(Sender: TObject);
begin
 NextPhase := NextPhase - 1;
 NextProcs[NextPhase];
end;

procedure TLoadBitmapWizard.NextButtonClick(Sender: TObject);
begin
 NextPhase := NextPhase + 1;
 NextProcs[NextPhase];
end;

procedure TLoadBitmapWizard.DrawColorConvertImage;
var
 x, y, i: Integer;
 TempBitmap: TBitmap;
begin
 TempBitmap := TBitmap.Create;
 ColorConvertImage.Width := SourceBitmap.Width * 2;
 with TempBitmap do
  begin
   Height := SourceBitmap.Height;
   Width  := SourceBitmap.Width;
   Canvas.Draw(0, 0, SourceBitmap);
   for x := 0 to Width - 1 do
    for y := 0 to Height - 1 do
     for i := 0 to ColorList.Count - 1 do
      if Canvas.Pixels[x, y] = ColorList.Items[i].Color then
       begin
        Canvas.Pixels[x, y] := MuseColors[ColorList.Items[i].MuseColorIndex];
        Break;
       end;
  end;
 ColorConvertImage.Picture.Bitmap.Assign(TempBitmap);
 TempBitmap.Free;
end;

procedure TLoadBitmapWizard.NextProc1;
begin
 FileLoadNotebook.ActivePage := 'FileName';
 FileName := StoredFileName;
 BackButton.Enabled := False;
 NextButton.Caption := '&Next >';
 if not StayFileNamePage and NextButton.Enabled then
  begin
   StayFileNamePage := True;
   NextPhase := NextPhase + 1;
   NextProcs[NextPhase];
  end;
end;

procedure TLoadBitmapWizard.NextProc2;
begin
 FileLoadNotebook.ActivePage := 'Stretch';
 BackButton.Enabled := True;
 NextButton.Caption := '&Next >';
 StoredFileName := FileName;
 Modified := BitmapHeight <> PreviewImage.Picture.Bitmap.Height;
 with CutoffImage.Picture.Bitmap do
  begin
   Height := 0;
   Height := BitmapHeight;
   Width := PreviewImage.Picture.Bitmap.Width;
   Canvas.Draw(0, 0, PreviewImage.Picture.Bitmap);
   CutoffImage.Width := Width * 2;
  end;
 if PreviewImage.Picture.Bitmap.Height <= BitmapHeight then
  begin
   CutoffRadioButton.Checked := True;
   NextPhase := NextPhase + 1;
   NextProcs[NextPhase];
   Exit;
  end;
 with ReduceImage.Picture.Bitmap do
  begin
   Height := 0;
   Height := BitmapHeight;
   Width := Round(PreviewImage.Picture.Bitmap.Width *
                    (BitmapHeight /
                      PreviewImage.Picture.Bitmap.Height));
   Canvas.StretchDraw(Rect(0, 0, Width, Height), PreviewImage.Picture.Bitmap);
   ReduceImage.Width := Width * 2;
  end;
 with ReduceVImage.Picture.Bitmap do
  begin
   Height := 0;
   Height := BitmapHeight;
   Width := PreviewImage.Picture.Bitmap.Width;
   Canvas.StretchDraw(Rect(0, 0, Width, Height), PreviewImage.Picture.Bitmap);
   ReduceVImage.Width := Width * 2;
  end;
end;

procedure TLoadBitmapWizard.NextProc3;
var
 x, y, i: Integer;
begin
 FileLoadNotebook.ActivePage := 'ColorConvert';
 BackButton.Enabled := True;
 NextButton.Caption := '&Finish';
 if CutoffRadioButton.Checked then
  SourceBitmap := CutoffImage.Picture.Bitmap;
 if ReduceRadioButton.Checked then
  SourceBitmap := ReduceImage.Picture.Bitmap;
 if ReduceVRadioButton.Checked then
  SourceBitmap := ReduceVImage.Picture.Bitmap;
 ColorList.Clear;
 try
  with SourceBitmap do
   for x := 0 to Width - 1 do
    for y := 0 to Height - 1 do
     ColorList.Add(Canvas.Pixels[x, y]);
 except
  on EColorListError do
   begin
    with ColorConvertImage.Picture.Bitmap do
     begin
      Height := SourceBitmap.Height;
      Width  := SourceBitmap.Width;
      for x := 0 to Width - 1 do
       for y := 0 to Height - 1 do
        Canvas.Pixels[x, y] :=
           MuseColors[ColorToMuseColorIndex(SourceBitmap.Canvas.Pixels[x, y])];
     end;
    NextPhase := NextPhase + 1;
    NextProcs[NextPhase];
    Exit;
   end;
 end;
 if 0 = ColorList.Count then
  begin
   ColorConvertImage.Picture.Bitmap.Assign(SourceBitmap);
   NextPhase := NextPhase + 1;
   NextProcs[NextPhase];
   Exit;
  end;
 Modified := True;
 ColorList.Sort;
 with ColorConvertlistBox.Items do
  begin
   BeginUpdate;
   Clear;
   for i := 0 to ColorList.Count - 1 do
    Add(ColorList.DisplayString(i));
   EndUpdate;
  end;
 DrawColorConvertImage;
end;

procedure TLoadBitmapWizard.NextProc4;
begin
 ModalResult := mrOk;
end;

procedure TLoadBitmapWizard.SetFileName(AFileName: TFileName);
begin
 if FFileName = AFileName then   Exit;
 FFileName := AFileName;
 FileNameEdit.Text := AFileName;
 if not FileExists(FileNameEdit.Text) then
  begin
   PreviewImage.Width := 0;
   Exit;
  end;
 try
  PreviewImage.Picture.Bitmap.LoadFromFile(FileName);
 except
  on EInvalidGraphic do
   begin
    NextButton.Enabled := False;
    Exit;
   end;
 end;
 with PreviewImage.Picture.Bitmap do
  begin
   if 0 = Height then   Height := 1;
   if 0 = Width  then   Width  := 1;
  end;
 NextButton.Enabled := True;
 PreviewImage.Width := Round(PreviewImage.Picture.Bitmap.Width *
                              (BitmapHeight /
                                PreviewImage.Picture.Bitmap.Height) * 2);
 BitmapOpenDialog.FileName := ExtractFileName(FileName);
 BitmapOpenDialog.InitialDir := ExtractFilePath(FileName);
end;

function TLoadBitmapWizard.Execute(AFileName: TFileName): Boolean;
begin
 Modified := False;
 StayFileNamePage := '' = AFileName;
 StoredFileName := AFileName;
 Result := mrOk = ShowModal;
end;

function TLoadBitmapWizard.NotesBitmap: TBitmap;
begin
 Result := ColorConvertImage.Picture.Bitmap;
end;

end.
