unit KeybrdF;

interface

uses
  Classes, Controls, ExtCtrls, Forms, Graphics;

type
  TKeyboardForm = class(TForm)
    KeyboardTimer: TTimer;
    TopPanel: TPanel;
    KeyboardImage: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure KeyboardTimerTimer(Sender: TObject);
    procedure KeyboardImageClick(Sender: TObject);
  protected
    CurrentRow: Integer;
    KeyboardBitmap: TBitmap;
    KeyColors: array[0..55] of TColor;
    Playing: Boolean;
    procedure Play;
    procedure RefreshKeyboard;
    procedure Stop;
  end;

var
  KeyboardForm: TKeyboardForm;

implementation

uses
  MsPaintF;

{$R *.DFM}

procedure TKeyboardForm.FormCreate(Sender: TObject);
var
 i: Integer;
begin
 KeyboardBitmap := TBitmap.Create;
 with KeyboardBitmap do
  begin
   Height := KeyboardImage.Height;
   Width := KeyboardImage.Width;
   with Canvas do
    begin
     Brush.Style := bsSolid;
     Pen.Color := clBlack;
     Pen.Mode := pmCopy;
     Pen.Style := psSolid;
     Pen.Width := 1;
    end;
  end;
 KeyboardImage.Picture.Bitmap.Height := KeyboardImage.Height;
 KeyboardImage.Picture.Bitmap.Width := KeyboardImage.Width;
 for i := 0 to 55 do
  KeyColors[i] := clWhite;
 RefreshKeyboard;
end;

procedure TKeyboardForm.FormDestroy(Sender: TObject);
begin
 KeyboardBitmap.Free;
end;

procedure TKeyboardForm.KeyboardTimerTimer(Sender: TObject);
var
 i: Integer;
begin
 with MusPaintForm.NotesBitmap do
  begin
   if Width - 1 < CurrentRow then
    begin
     Stop;
     Exit;
    end;
   for i := 0 to 55 do
    KeyColors[i] := Canvas.Pixels[CurrentRow, 55 - i];
   RefreshKeyboard;
   CurrentRow := CurrentRow + 1;
  end;
end;

procedure TKeyboardForm.KeyboardImageClick(Sender: TObject);
begin
 Playing := not Playing;
 if Playing then
  Stop
 else
  Play;
end;

procedure TKeyboardForm.Play;
var
 i: Integer;
begin
 for i := 0 to 55 do
  KeyColors[i] := MusPaintForm.NotesBitmap.Canvas.Pixels[0, 55 - i];
 RefreshKeyboard;
 CurrentRow := 1;
 with MusPaintForm do
  KeyboardTimer.Interval := Trunc(60000 / Tempo / 16 * NoteLength);
 KeyboardTimer.Enabled := True;
end;

procedure TKeyboardForm.RefreshKeyboard;
var
 i: Integer;
begin
 with KeyboardBitmap.Canvas do
  begin
   for i := 0 to 55 do
    begin
     Brush.Color := KeyColors[i];
     Rectangle(i * 10 - 1, 0, i * 10 + 10, 57);
    end;
   Brush.Color := clGray;
   for i := 0 to 55 do
    if not (i mod 7 in [3, 6]) then
     Rectangle(i * 10 + 7, 0, i * 10 + 13, 37);
  end;
 KeyboardImage.Picture.Bitmap.Canvas.Draw(0, 0, KeyboardBitmap);
end;

procedure TKeyboardForm.Stop;
var
 i: Integer;
begin
 KeyboardTimer.Enabled := False;
 for i := 0 to 55 do
  KeyColors[i] := clWhite;
 RefreshKeyboard;
end;

end.
