unit MsPaintF;

interface

uses
  Buttons, Classes, Controls, Dialogs, ExtCtrls, FileCtrl, Forms, Menus,
  Messages, ShellAPI, StdCtrls, SysUtils, TabNotBk, WinProcs, WinTypes,
  Graphics,
  BmpData, BmpLoadW, CnvsSizD, KeybrdF, NoteLenD, TempoD, VersionD;

const
  VersionStr = '1.2';
  DateStr = '2001/03/10';

const
  BitmapHeight     = 56;{Pixels}{=NotesCount}
  BottomBlanc      = 25;{Pixels}
  CellHeight       =  5;{Pixels}
  ColorsCount      = 16;{0:BackColor,1:A,...}
  EndBlanc         = 63;{Pixels}
  NoteLengthsCount =  7;{0:Dialog,1:64noe,2:32note,...}
  NotesCount       = 56;{o0f through 08e}
  NoteSystemsCount =  2;{0:x0,...}
  PixelsPer64Note  =  2;{Pixels}
  TopBlanc         = 25;{Pixels}
  LineSpace        = 32;{Pixels}{=PixelsPer64Note*16}

const
  GridColor  = clSilver;
  BorderColor = clBlack;
  MusicLineColor = clBlack;
  AdditionalLineColor = clGray;
  BeatLineColor = clSilver;

const
  MusicHeight = 331;
  MusicLWidth =  72;

const
  MuseColors: array[0..ColorsCount] of TColor =
    ($00FFFFFF,
     $006000FF, $006080FF, $0040C0FF, $0080FFFF,
     $0040FFC0, $0040FF20, $0000C000, $00408040,
     $00A0C000, $00FFE000, $00FFA000, $00FF6020,
     $00FF60A0, $00FF60FF, $00C020FF, $004060A0);
  MuseColorNames: array[0..ColorsCount] of Char =
    ('W','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','Z');

const
  NoteSystemNames: array[0..NoteSystemsCount] of String[2] =
    ('x0', 'x1', 'x2');

const
  NoteNodes: array[0..NoteSystemsCount, 0..NotesCount] of String[3] =
    (('',                  'o0f', 'o0s', 'o0l', 'o0c',
      'o1d', 'o1r', 'o1m', 'o1f', 'o1s', 'o1l', 'o1c',
      'o2d', 'o2r', 'o2m', 'o2f', 'o2s', 'o2l', 'o2c',
      'o3d', 'o3r', 'o3m', 'o3f', 'o3s', 'o3l', 'o3c',
      'o4d', 'o4r', 'o4m', 'o4f', 'o4s', 'o4l', 'o4c',
      'o5d', 'o5r', 'o5m', 'o5f', 'o5s', 'o5l', 'o5c',
      'o6d', 'o6r', 'o6m', 'o6f', 'o6s', 'o6l', 'o6c',
      'o7d', 'o7r', 'o7m', 'o7f', 'o7s', 'o7l', 'o7c',
      'o8d', 'o8r', 'o8m'                            ),
     ('',                  'o0f', 'o0g', 'o0a', 'o0b',
      'o1c', 'o1d', 'o1e', 'o1f', 'o1g', 'o1a', 'o1b',
      'o2c', 'o2d', 'o2e', 'o2f', 'o2g', 'o2a', 'o2b',
      'o3c', 'o3d', 'o3e', 'o3f', 'o3g', 'o3a', 'o3b',
      'o4c', 'o4d', 'o4e', 'o4f', 'o4g', 'o4a', 'o4b',
      'o5c', 'o5d', 'o5e', 'o5f', 'o5g', 'o5a', 'o5b',
      'o6c', 'o6d', 'o6e', 'o6f', 'o6g', 'o6a', 'o6b',
      'o7c', 'o7d', 'o7e', 'o7f', 'o7g', 'o7a', 'o7b',
      'o8c', 'o8d', 'o8e'                            ),
     ('',                  'o0f', 'o0g', 'o0a', 'o0h',
      'o1c', 'o1d', 'o1e', 'o1f', 'o1g', 'o1a', 'o1h',
      'o2c', 'o2d', 'o2e', 'o2f', 'o2g', 'o2a', 'o2h',
      'o3c', 'o3d', 'o3e', 'o3f', 'o3g', 'o3a', 'o3h',
      'o4c', 'o4d', 'o4e', 'o4f', 'o4g', 'o4a', 'o4h',
      'o5c', 'o5d', 'o5e', 'o5f', 'o5g', 'o5a', 'o5h',
      'o6c', 'o6d', 'o6e', 'o6f', 'o6g', 'o6a', 'o6h',
      'o7c', 'o7d', 'o7e', 'o7f', 'o7g', 'o7a', 'o7h',
      'o8c', 'o8d', 'o8e'                            ));

const
  ToolsCount = 5;
type
  TTool = (tlPen, tlFill, tlLine, tlAngle, tlEllipse, tlSelect);
const
  ToolNames: array[0..ToolsCount] of String =
    ('y', 'tB', '', 'lp', 'ȉ~', 'I');

type
  TMusPaintForm = class(TForm)
    MusPaintMainMenu: TMainMenu;
     FileCommandGroup: TMenuItem;
      NewBitmapCommand: TMenuItem;
      LoadBitmapCommand: TMenuItem;
      N1: TMenuItem;
      SaveBitmapAsCommand: TMenuItem;
      SaveBitmapCommand: TMenuItem;
      N2: TMenuItem;
      SaveMusAsCommand: TMenuItem;
      SaveMusCommand: TMenuItem;
      N3: TMenuItem;
      ExitCommand: TMenuItem;
     EditCommandGroup: TMenuItem;
      UndoCommand: TMenuItem;
      N4: TMenuItem;
      DeleteCommand: TMenuItem;
      SelectAllCommand: TMenuItem;
      N5: TMenuItem;
      CopyMuseDataCommand: TMenuItem;
      N13: TMenuItem;
      SetCanvasSizeCommand: TMenuItem;
      NoteLengthCommandGroup: TMenuItem;
       NoteLength64Command: TMenuItem;
       NoteLength32Command: TMenuItem;
       NoteLength16Command: TMenuItem;
       NoteLength8Command: TMenuItem;
       NoteLength4Command: TMenuItem;
       NoteLength2Command: TMenuItem;
       NoteLength1Command: TMenuItem;
       N9: TMenuItem;
       SetNoteLengthCommand: TMenuItem;
      NoteSystemCommandGroup: TMenuItem;
       x0Command: TMenuItem;
       x1Command: TMenuItem;
       x2Command: TMenuItem;
      TempoCommand: TMenuItem;
      N12: TMenuItem;
      CheckSaveCommand: TMenuItem;
     DisplayCommandGroup: TMenuItem;
      ShowToolBoxCommand: TMenuItem;
      ShowColorBoxCommand: TMenuItem;
      ShowStatusBarCommand: TMenuItem;
      N6: TMenuItem;
      ZoomUpCommand: TMenuItem;
      ZoomNormalCommand: TMenuItem;
      N7: TMenuItem;
      ShowGridCommand: TMenuItem;
     ToolCommandGroup: TMenuItem;
      tlPenCommand: TMenuItem;
      tlFillCommand: TMenuItem;
      tlLineCommand: TMenuItem;
      tlAngleCommand: TMenuItem;
      tlEllipseCommand: TMenuItem;
      tlSelectCommand: TMenuItem;
      N8: TMenuItem;
      bsClearCommand: TMenuItem;
      bsSolidCommand: TMenuItem;
     ColorCommandGroup: TMenuItem;
      ColorACommand: TMenuItem;
      ColorBCommand: TMenuItem;
      ColorCCommand: TMenuItem;
      ColorDCommand: TMenuItem;
      ColorECommand: TMenuItem;
      ColorFCommand: TMenuItem;
      ColorGCommand: TMenuItem;
      ColorHCommand: TMenuItem;
      ColorICommand: TMenuItem;
      ColorJCommand: TMenuItem;
      ColorKCommand: TMenuItem;
      ColorLCommand: TMenuItem;
      ColorMCommand: TMenuItem;
      ColorNCommand: TMenuItem;
      ColorOCommand: TMenuItem;
      ColorZCommand: TMenuItem;
     HelpCommandGroup: TMenuItem;
      ShowManualCommand: TMenuItem;
      N10: TMenuItem;
      VersionCommand: TMenuItem;
    ToolBoxPanel: TPanel;
     ToolsBasePanel: TPanel;
      tlPenSpeedButton: TSpeedButton;
      tlFillSpeedButton: TSpeedButton;
      tlLineSpeedButton: TSpeedButton;
      tlAngleSpeedButton: TSpeedButton;
      tlEllipseSpeedButton: TSpeedButton;
      tlSelectSpeedButton: TSpeedButton;
     BrushStyleBasePanel: TPanel;
      bsSolidSpeedButton: TSpeedButton;
      bsClearSpeedButton: TSpeedButton;
    ColorBoxPanel: TPanel;
     SelColorsPanel: TPanel;
      FrontColorPanel: TPanel;
      BackColorPanel: TPanel;
     ColorAPanel: TPanel;
     ColorBPanel: TPanel;
     ColorCPanel: TPanel;
     ColoDPanel: TPanel;
     ColorEPanel: TPanel;
     ColorFPanel: TPanel;
     ColorGPanel: TPanel;
     ColorHPanel: TPanel;
     ColorIPanel: TPanel;
     ColorJPanel: TPanel;
     ColorKPanel: TPanel;
     ColorLPanel: TPanel;
     ColorMPanel: TPanel;
     ColorNPanel: TPanel;
     ColorOPanel: TPanel;
     ColorZPanel: TPanel;
    StatusBarPanel: TPanel;
     BitmapSizePanel: TPanel;
     NoteLengthPanel: TPanel;
     NoteSystemPanel: TPanel;
     TempoPanel: TPanel;
     MagnificationPanel: TPanel;
     ShowGridPanel: TPanel;
     ToolPanel: TPanel;
     BrushStylePanel: TPanel;
     MousePointPanel: TPanel;
     SelSizePanel: TPanel;
     SelColorPanel: TPanel;
    MusicBasePanel: TPanel;
     HorizontalBasePanel: TPanel;
      ShowGSpeedButton: TSpeedButton;
      ShowTSpeedButton: TSpeedButton;
      ShowNotASpeedButton: TSpeedButton;
      ShowASpeedButton: TSpeedButton;
      HorizontalScrollBar: TScrollBar;
     MusicScrollBox: TScrollBox;
      MusicLImage: TImage;
      MusicPanel: TPanel;
       MusicImage: TImage;
       SelectImage: TImage;
    SaveBitmapDialog: TSaveDialog;
    SaveMusDialog: TSaveDialog;
    NoteLengthPopupMenu: TPopupMenu;
     NoteLength64PCommand: TMenuItem;
     NoteLength32PCommand: TMenuItem;
     NoteLength16PCommand: TMenuItem;
     NoteLength8PCommand: TMenuItem;
     NoteLength4PCommand: TMenuItem;
     NoteLength2PCommand: TMenuItem;
     NoteLength1PCommand: TMenuItem;
     N11: TMenuItem;
     SetNoteLengthPCommand: TMenuItem;
    ColorPopupMenu: TPopupMenu;
     ColorAPCommand: TMenuItem;
     ColorBPCommand: TMenuItem;
     ColorCPCommand: TMenuItem;
     ColorDPCommand: TMenuItem;
     ColorEPCommand: TMenuItem;
     ColorFPCommand: TMenuItem;
     ColorGPCommand: TMenuItem;
     ColorHPCommand: TMenuItem;
     ColorIPCommand: TMenuItem;
     ColorJPCommand: TMenuItem;
     ColorKPCommand: TMenuItem;
     ColorLPCommand: TMenuItem;
     ColorMPCommand: TMenuItem;
     ColorNPCommand: TMenuItem;
     ColorOPCommand: TMenuItem;
     ColorZPCommand: TMenuItem;
    N14: TMenuItem;
    ShowKeyboardCommand: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure NewBitmapCommandClick(Sender: TObject);
    procedure LoadBitmapCommandClick(Sender: TObject);
    procedure SaveBitmapAsCommandClick(Sender: TObject);
    procedure SaveBitmapCommandClick(Sender: TObject);
    procedure SaveMusAsCommandClick(Sender: TObject);
    procedure SaveMusCommandClick(Sender: TObject);
    procedure ExitCommandClick(Sender: TObject);
    procedure UndoCommandClick(Sender: TObject);
    procedure DeleteCommandClick(Sender: TObject);
    procedure SelectAllCommandClick(Sender: TObject);
    procedure CopyMuseDataCommandClick(Sender: TObject);
    procedure SetCanvasSizeCommandClick(Sender: TObject);
    procedure NoteLengthCommandsClick(Sender: TObject);
    procedure SetNoteLengthCommandClick(Sender: TObject);
    procedure NoteSystemCommandsClick(Sender: TObject);
    procedure TempoCommandClick(Sender: TObject);
    procedure CheckSaveCommandClick(Sender: TObject);
    procedure ShowToolBoxCommandClick(Sender: TObject);
    procedure ShowColorBoxCommandClick(Sender: TObject);
    procedure ShowStatusBarCommandClick(Sender: TObject);
    procedure ZoomUpCommandClick(Sender: TObject);
    procedure ZoomNormalCommandClick(Sender: TObject);
    procedure ShowGridCommandClick(Sender: TObject);
    procedure ToolCommandsClick(Sender: TObject);
    procedure BrushStyleCommandsClick(Sender: TObject);
    procedure ColorCommandsClick(Sender: TObject);
    procedure ShowManualCommandClick(Sender: TObject);
    procedure VersionCommandClick(Sender: TObject);
    procedure ToolSpeedButtonsClick(Sender: TObject);
    procedure BrushStyleSpeedButtonsClick(Sender: TObject);
    procedure ColorBoxPanelResize(Sender: TObject);
    procedure ColorPanelsClick(Sender: TObject);
    procedure BitmapSizePanelDblClick(Sender: TObject);
    procedure NoteLengthPanelMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure NoteSystemPanelDblClick(Sender: TObject);
    procedure TempoPanelDblClick(Sender: TObject);
    procedure MagnificationPanelDblClick(Sender: TObject);
    procedure ShowGridPanelDblClick(Sender: TObject);
    procedure ToolPanelDblClick(Sender: TObject);
    procedure BrushStylePanelDblClick(Sender: TObject);
    procedure SelColorPanelDblClick(Sender: TObject);
    procedure HorizontalScrollBarScroll(Sender: TObject;
      ScrollCode: TScrollCode; var ScrollPos: Integer);
    procedure HorizontalScrollBarChange(Sender: TObject);
    procedure MusicScrollBoxResize(Sender: TObject);
    procedure MusicPanelResize(Sender: TObject);
    procedure MusicImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MusicImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure MusicImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SelectImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SelectImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SelectImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure NCMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure NCMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ShowASpeedButtonClick(Sender: TObject);
    procedure ShowKeyboardCommandClick(Sender: TObject);
  private
    FBackColorIndex: Integer;
    FBitmapFileName: TFileName;
    FBitmapWidth: Integer;
    FBrushStyle: TBrushStyle;
    FCellWidth: Integer;
    FCheckSave: Boolean;
    FFrontColorIndex: Integer;
    FMagnification: Integer;
    FModified: Boolean;
    FMusFileName: TFileName;
    FNoteLength: Integer;
    FNoteSystem: Integer;
    FOffset: Integer;
    FSelected: Boolean;
    FSelImagePos: TRect;
    FSelRect: TRect;
    FShowGrid: Boolean;
    FTempo: Integer;
    FTool: TTool;
    ShowASpeedButtonDown: Boolean;
  protected
    ColorCommands: array[1..ColorsCount] of TMenuItem;
    ColorPCommands: array[1..ColorsCount] of TMenuItem;
    FigureBitmap: TBitmap;
    MouseButtonDown: Boolean;
    MouseDownPoint: TPoint;
    MouseLButtonDown: Boolean;
    MusicBitmap: TBitmap;
    NoteCommands: array[0..NoteLengthsCount] of TMenuItem;
    NotePCommands: array[0..NoteLengthsCount] of TMenuItem;
    NoteSystemCommands: array[0..NoteSystemsCount] of TMenuItem;
    Refreshing: Boolean;
    SelectBitmap: TBitmap;
    SelectedColor: TColor;
    ToolCommands: array[0..ToolsCount] of TMenuItem;
    ToolSpeedButtons: array[0..ToolsCount] of TSpeedButton;
    UndoBitmap: TBitmap;
    WorkBitmap: TBitmap;
    function GetBackColor: TColor;
    function GetFrontColor: TColor;
    procedure SetBitmapFileName(AFileName: TFileName);
    procedure SetBitmapWidth(AWidth: Integer);
    procedure SetBrushStyle(ABrushStyle: TBrushStyle);
    procedure SetCellWidth(AWidth: Integer);
    procedure SetCheckSave(ACheck: Boolean);
    procedure SetFrontColorIndex(AIndex: Integer);
    procedure SetMagnification(AMagnification: Integer);
    procedure SetModified(AModified: Boolean);
    procedure SetMusFileName(AFileName: TFileName);
    procedure SetNoteLength(ALength: Integer);
    procedure SetNoteSystem(ASystem: Integer);
    procedure SetOffset(AOffset: Integer);
    procedure SetSelected(ASelected: Boolean);
    procedure SetSelImagePos(APos: TRect);
    procedure SetSelRect(ARect: TRect);
    procedure SetShowGrid(AGrid: Boolean);
    procedure SetTempo(ATempo: Integer);
    procedure SetTool(ATool: TTool);
    procedure WMDropFiles(var Message: TWMDropFiles); message WM_DROPFILES;
  public
    NotesBitmap: TBitmap;
    property BackColor: TColor read GetBackColor;
    property BackColorIndex: Integer read FBackColorIndex;
    property BitmapFileName: TFileName read FBitmapFileName
                                      write SetBitmapFileName;
    property BitmapWidth: Integer read FBitmapWidth write SetBitmapWidth;
    property BrushStyle: TBrushStyle read FBrushStyle
                                        write SetBrushStyle;
    property CellWidth: Integer read FCellWidth write SetCellWidth;
    property CheckSave: Boolean read FCheckSave write SetCheckSave;
    property FrontColor: TColor read GetFrontColor;
    property FrontColorIndex: Integer read FFrontColorIndex
                                     write SetFrontColorIndex;
    property Magnification: Integer read FMagnification
                                   write SetMagnification;
    property Modified: Boolean read FModified write SetModified;
    property MusFileName: TFileName read FMusFileName write SetMusFileName;
    property NoteLength: Integer read FNoteLength write SetNoteLength;
    property NoteSystem: Integer read FNoteSystem write SetNoteSystem;
    property Offset: Integer read FOffset write SetOffset;
    property Selected: Boolean read FSelected write SetSelected;
    property SelImagePos: TRect read FSelImagePos write SetSelImagePos;
    property SelRect: TRect read FSelRect write SetSelRect;
    property ShowGrid: Boolean read FShowGrid write SetShowGrid;
    property Tempo: Integer read FTempo write SetTempo;
    property Tool: TTool read FTool write SetTool;
    procedure ArrangeImages;
    procedure DrawMusic(var NotesBitmap: TBitmap);
    procedure DrawSelectImage;
    procedure LoadBitmap(AFileName: TFileName);
    procedure NewBitmap;
    function MusicPointToBitmapPoint(APoint: TPoint): TPoint;
    function MusicRectToBitmapRect(ARect: TRect): TRect;
    function BitmapPointToMusicPoint(APoint: TPoint): TPoint;
    function BitmapRectToMusicRect(ARect: TRect): TRect;
    function SaveBitmap: Boolean;
    function SaveBitmapAs: Boolean;
    procedure SaveMus;
    procedure SaveMusAs;
    procedure SaveMusFile;
  end;

function Rect(ALeft, ATop, ARight, ABottom: Integer): TRect;
procedure RectangleR(Canvas: TCanvas; ARect: TRect);
procedure EllipseR(Canvas: TCanvas; ARect: TRect);

var
  MusPaintForm: TMusPaintForm;

implementation

{$R *.DFM}

function Rect(ALeft, ATop, ARight, ABottom: Integer): TRect;
begin
 if ALeft < ARight then
  begin
   Result.Left := ALeft;
   Result.Right := ARight;
  end
 else
  begin
   Result.Left := ARight;
   Result.Right := ALeft;
  end;
 if ATop < ABottom then
  begin
   Result.Top := ATop;
   Result.Bottom := ABottom;
  end
 else
  begin
   Result.Top := ABottom;
   Result.Bottom := ATop;
  end;
end;

procedure RectangleR(Canvas: TCanvas; ARect: TRect);
begin
 with ARect do
  Canvas.Rectangle(Left, Top, Right, Bottom);
end;

procedure EllipseR(Canvas: TCanvas; ARect: TRect);
begin
 with ARect do
  Canvas.Ellipse(Left, Top, Right, Bottom);
end;

procedure TMusPaintForm.FormCreate(Sender: TObject);
var
 i: Integer;
 TempBitmap: TBitmap;
begin
 DragAcceptFiles(Handle, TRUE);
 TempBitmap := TBitmap.Create;

 LoadBitmapWizard := TLoadBitmapWizard.Create(Self);
 CanvasSizeDialog := TCanvasSizeDialog.Create(Self);
 NoteLengthDialog := TNoteLengthDialog.Create(Self);
 TempoSetDialog := TTempoSetDialog.Create(Self);
 VersionDialog := TVersionDialog.Create(Self);

 ColorCommands[1]  := ColorACommand;
 ColorCommands[2]  := ColorBCommand;
 ColorCommands[3]  := ColorCCommand;
 ColorCommands[4]  := ColorDCommand;
 ColorCommands[5]  := ColorECommand;
 ColorCommands[6]  := ColorFCommand;
 ColorCommands[7]  := ColorGCommand;
 ColorCommands[8]  := ColorHCommand;
 ColorCommands[9]  := ColorICommand;
 ColorCommands[10] := ColorJCommand;
 ColorCommands[11] := ColorKCommand;
 ColorCommands[12] := ColorLCommand;
 ColorCommands[13] := ColorMCommand;
 ColorCommands[14] := ColorNCommand;
 ColorCommands[15] := ColorOCommand;
 ColorCommands[16] := ColorZCommand;
 ColorPCommands[1]  := ColorAPCommand;
 ColorPCommands[2]  := ColorBPCommand;
 ColorPCommands[3]  := ColorCPCommand;
 ColorPCommands[4]  := ColorDPCommand;
 ColorPCommands[5]  := ColorEPCommand;
 ColorPCommands[6]  := ColorFPCommand;
 ColorPCommands[7]  := ColorGPCommand;
 ColorPCommands[8]  := ColorHPCommand;
 ColorPCommands[9]  := ColorIPCommand;
 ColorPCommands[10] := ColorJPCommand;
 ColorPCommands[11] := ColorKPCommand;
 ColorPCommands[12] := ColorLPCommand;
 ColorPCommands[13] := ColorMPCommand;
 ColorPCommands[14] := ColorNPCommand;
 ColorPCommands[15] := ColorOPCommand;
 ColorPCommands[16] := ColorZPCommand;
 NoteCommands[0] := SetNoteLengthCommand;
 NoteCommands[1] := NoteLength64Command;
 NoteCommands[2] := NoteLength32Command;
 NoteCommands[3] := NoteLength16Command;
 NoteCommands[4] := NoteLength8Command;
 NoteCommands[5] := NoteLength4Command;
 NoteCommands[6] := NoteLength2Command;
 NoteCommands[7] := NoteLength1Command;
 NotePCommands[0] := SetNoteLengthPCommand;
 NotePCommands[1] := NoteLength64PCommand;
 NotePCommands[2] := NoteLength32PCommand;
 NotePCommands[3] := NoteLength16PCommand;
 NotePCommands[4] := NoteLength8PCommand;
 NotePCommands[5] := NoteLength4PCommand;
 NotePCommands[6] := NoteLength2PCommand;
 NotePCommands[7] := NoteLength1PCommand;
 NoteSystemCommands[0] := x0Command;
 NoteSystemCommands[1] := x1Command;
 NoteSystemCommands[2] := x2Command;
 ToolCommands[0] := tlPenCommand;
 ToolCommands[1] := tlFillCommand;
 ToolCommands[2] := tlLineCommand;
 ToolCommands[3] := tlAngleCommand;
 ToolCommands[4] := tlEllipseCommand;
 ToolCommands[5] := tlSelectCommand;
 ToolSpeedButtons[0] := tlPenSpeedButton;
 ToolSpeedButtons[1] := tlFillSpeedButton;
 ToolSpeedButtons[2] := tlLineSpeedButton;
 ToolSpeedButtons[3] := tlAngleSpeedButton;
 ToolSpeedButtons[4] := tlEllipseSpeedButton;
 ToolSpeedButtons[5] := tlSelectSpeedButton;

 FigureBitmap := TBitmap.Create;
 MusicBitmap  := TBitmap.Create;
 NotesBitmap  := TBitmap.Create;
 SelectBitmap := TBitmap.Create;
 UndoBitmap   := TBitmap.Create;
 WorkBitmap   := TBitmap.Create;

 with WorkBitmap do
  begin
   Height := MusicHeight;
   Width := MusicLWidth;
   with Canvas do
    begin
     Brush.Color := BackColor;
     Brush.Style := bsSolid;
     FillRect(Rect(0, 0, Width, Height));
     MonoDataToBitmap(TempBitmap, GmarkBitmapData);
     Draw(10, 116, TempBitmap);
     MonoDataToBitmap(TempBitmap, FmarkBitmapData);
     Draw(11, 189, TempBitmap);
     MonoDataToBitmap(TempBitmap, FourBitmapData);
     Draw(47, 129, TempBitmap);
     Draw(47, 149, TempBitmap);
     Draw(47, 189, TempBitmap);
     Draw(47, 209, TempBitmap);
     Pen.Color := MusicLineColor;
     Pen.Mode := pmCopy;
     Pen.Style := psSolid;
     Pen.Width := 1;
     for i := 0 to 4 do
      begin
       MoveTo(6, 127 + i * 10);
       LineTo(72, 127 + i * 10);
       MoveTo(6, 128 + i * 10);
       LineTo(72, 128 + i * 10);
       MoveTo(6, 187 + i * 10);
       LineTo(72, 187 + i * 10);
       MoveTo(6, 188 + i * 10);
       LineTo(72, 188 + i * 10);
      end;
    end;
  end;
 MusicLImage.Picture.Bitmap.Assign(WorkBitmap);
 MonoDataToBitmap(FigureBitmap, FigureBitmapData);

 SaveBitmapDialog.InitialDir := ExtractFilePath(Application.ExeName);
 SaveMusDialog.InitialDir := ExtractFilePath(Application.ExeName);

 Refreshing := True;
 FBackColorIndex := 0;
 BitmapWidth := 64;
 BrushStyle := bsClear;
 CheckSave := True;
 FrontColorIndex := 1;
 Magnification := 1;
 MouseButtonDown := False;
 MouseLButtonDown := False;
 MusFileName := '';
 NoteLength := 2;
 NoteSystem := 1;
 Offset := 0;
 Selected := False;
 SelRect := Rect(0, 0, 0, 0);
 ShowASpeedButtonDown := False;
 ShowGrid := False;
 Tempo := 120;
 Tool := tlPen;
 Refreshing := False;
 Modified := False;

 if 0 < ParamCount then
  if FileExists(ParamStr(1)) then
   try
    TempBitmap.LoadFromFile(ParamStr(1));
    TempBitmap.Width := 0;
    LoadBitmap(ParamStr(1));
   except
    on EInvalidGraphic do
     begin
      TempBitmap.Free;
      Exit;
     end;
   end
  else
   NewBitmap
 else
  NewBitmap;

 TempBitmap.Free;
end;

procedure TMusPaintForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
 MR: Integer;
begin
 if Modified and CheckSave then
  begin
   MessageBeep(MB_ICONQUESTION);
   MR := MessageBox(Handle, 'rbg}bv̕ύXۑ܂H',
                     'MusPaint', MB_YESNOCANCEL);
   if IDYES = MR then
    if SaveBitmap then
     Action := caFree
    else
     Action := caNone
   else if IDNO = MR then
    Action := caFree
   else if IDCANCEL = MR then
    Action := caNone;
  end
 else
  Action := caFree;
end;

procedure TMusPaintForm.FormDestroy(Sender: TObject);
begin
 Refreshing := True;
 LoadBitmapWizard.Free;
 CanvasSizeDialog.Free;
 NoteLengthDialog.Free;
 TempoSetDialog.Free;
 VersionDialog.Free;
 FigureBitmap.Free;
 MusicBitmap.Free;
 NotesBitmap.Free;
 SelectBitmap.Free;
 UndoBitmap.Free;
 WorkBitmap.Free;
end;

procedure TMusPaintForm.FormResize(Sender: TObject);
begin
 if Height < 256 then   Height := 256;
 if Width  < 200 then   Width  := 200;
end;

procedure TMusPaintForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if [] = Shift then
  case Key of
   $41{VK_A}..$4F{VK_O} : FrontColorIndex := Key - $40;
   $5A{VK_Z} : FrontColorIndex := 16;
  end;
 if [ssCtrl] = Shift then
  case Key of
   $21{VK_PRIOR} : Magnification := 1;
   $22{VK_NEXT}  : Magnification := 2;
  end;
end;

procedure TMusPaintForm.NewBitmapCommandClick(Sender: TObject);
begin
 NewBitmap;
end;

procedure TMusPaintForm.LoadBitmapCommandClick(Sender: TObject);
begin
 LoadBitmap('');
end;

procedure TMusPaintForm.SaveBitmapAsCommandClick(Sender: TObject);
begin
 SaveBitmapAs;
end;

procedure TMusPaintForm.SaveBitmapCommandClick(Sender: TObject);
begin
 SaveBitmap;
end;

procedure TMusPaintForm.SaveMusAsCommandClick(Sender: TObject);
begin
 SaveMusAs;
end;

procedure TMusPaintForm.SaveMusCommandClick(Sender: TObject);
begin
 SaveMus;
end;

procedure TMusPaintForm.ExitCommandClick(Sender: TObject);
begin
 Close;
end;

procedure TMusPaintForm.UndoCommandClick(Sender: TObject);
begin
 NotesBitmap.Assign(UndoBitmap);
 DrawMusic(NotesBitmap);
 UndoCommand.Enabled := False;
end;

procedure TMusPaintForm.DeleteCommandClick(Sender: TObject);
begin
 with SelectBitmap do with Canvas do
  begin
   Brush.Color := BackColor;
   Brush.Style := bsSolid;
   FillRect(Rect(0, 0, Width, Height));
  end;
 Selected := False;
end;

procedure TMusPaintForm.SelectAllCommandClick(Sender: TObject);
begin
 Selected := False;
 Tool := tlSelect;
 Selected := True;
 SelRect := Rect(0, 0, NotesBitmap.Width, NotesBitmap.Height);
 if bsClear = BrushStyle then
  with NotesBitmap.Canvas do
   begin
    Brush.Color := BackColor;
    Brush.Style := bsSolid;
    FillRect(SelRect);
    DrawMusic(NotesBitmap);
   end;
end;

procedure TMusPaintForm.CopyMuseDataCommandClick(Sender: TObject);
  function GetTempFileName: TFileName;
  var
   i: Longint;
   F: TFileName;
  begin
   Result := '';
   for i := 0 to 99999999 do
    begin
     F := ExtractFilePath(Application.ExeName) + '\' + IntToStr(i) + '.tmp';
     if not FileExists(F) then   Break;
    end;
   if not FileExists(F) then
    Result := F;
  end;
var
 BackMusFileName: TFileName;
begin
 try
  BackMusFileName := MusFileName;
  MusFileName := GetTempFileName;
  if '' = MusFileName then
   MessageBox(Handle, 'ꎞt@C쐬ł܂', 'MusPaint', MB_OK)
  else
   begin
    SaveMusFile;
    with TMemo.Create(Self) do
     begin
      Parent := Self;
      Visible := False;
      Lines.LoadFromFile(MusFileName);
      SelectAll;
      CopyToClipboard;
      Free;
     end;
   end;
 finally
  DeleteFile(MusFileName);
  MusFileName := BackMusFileName;
 end;
end;

procedure TMusPaintForm.SetCanvasSizeCommandClick(Sender: TObject);
begin
 if CanvasSizeDialog.Execute then
  BitmapWidth := CanvasSizeDialog.BitmapWidth;
end;

procedure TMusPaintForm.NoteLengthCommandsClick(Sender: TObject);
begin
 NoteLength := TMenuItem(Sender).Tag;
end;

procedure TMusPaintForm.SetNoteLengthCommandClick(Sender: TObject);
begin
 if NoteLengthDialog.Execute then
  NoteLength := NoteLengthDialog.NoteLength;
end;

procedure TMusPaintForm.NoteSystemCommandsClick(Sender: TObject);
begin
 NoteSystem := TMenuItem(Sender).Tag;
end;

procedure TMusPaintForm.TempoCommandClick(Sender: TObject);
begin
 Tempo := TempoSetDialog.Execute(Tempo);
end;

procedure TMusPaintForm.CheckSaveCommandClick(Sender: TObject);
begin
 CheckSave := not CheckSave;
end;

procedure TMusPaintForm.ShowToolBoxCommandClick(Sender: TObject);
var
 ShowToolBox: Boolean;
begin
 ShowToolBox := not ShowToolBoxCommand.Checked;
 ShowToolBoxCommand.Checked := ShowToolBox;
 ToolBoxPanel.Visible := ShowToolBox;
end;

procedure TMusPaintForm.ShowColorBoxCommandClick(Sender: TObject);
var
 ShowColorBox: Boolean;
begin
 ShowColorBox := not ShowColorBoxCommand.Checked;
 ShowColorBoxCommand.Checked := ShowColorBox;
 ColorBoxPanel.Visible := ShowColorBox;
 ColorBoxPanel.Top := StatusBarPanel.Top - ColorBoxPanel.Height - 1;
end;

procedure TMusPaintForm.ShowStatusBarCommandClick(Sender: TObject);
var
 ShowStatusBar: Boolean;
begin
 ShowStatusBar := not ShowStatusBarCommand.Checked;
 ShowStatusBarCommand.Checked := ShowStatusBar;
 StatusBarPanel.Visible := ShowStatusBar;
 ColorBoxPanel.Top := StatusBarPanel.Top - ColorBoxPanel.Height - 1;
end;

procedure TMusPaintForm.ZoomUpCommandClick(Sender: TObject);
begin
 Magnification := 2;
end;

procedure TMusPaintForm.ZoomNormalCommandClick(Sender: TObject);
begin
 Magnification := 1;
end;

procedure TMusPaintForm.ShowGridCommandClick(Sender: TObject);
begin
 ShowGrid := not ShowGrid;
end;

procedure TMusPaintForm.ToolCommandsClick(Sender: TObject);
begin
 Tool := TTool(TMenuItem(Sender).Tag);
end;

procedure TMusPaintForm.BrushStyleCommandsClick(Sender: TObject);
begin
 BrushStyle := TBrushStyle(TMenuItem(Sender).Tag);
end;

procedure TMusPaintForm.ColorCommandsClick(Sender: TObject);
begin
 FrontColorIndex := TMenuItem(Sender).Tag;
end;

procedure TMusPaintForm.ShowManualCommandClick(Sender: TObject);
var
 ReadMeTxt, Directory: array[0..80] of Char;
begin
 StrPCopy(ReadMeTxt, ExtractFilePath(Application.ExeName) + 'ReadMe.txt');
 StrPCopy(Directory, ExtractFilePath(Application.ExeName));
 ShellExecute(Handle, 'open', ReadMeTxt, nil, Directory, SW_SHOW);
end;

procedure TMusPaintForm.VersionCommandClick(Sender: TObject);
begin
 with VersionDialog do
 begin
  VersionLabel.Caption := 'Version ' + VersionStr;
  DateLabel.Caption := DateStr;
  MessageBeep(MB_ICONASTERISK);
  ShowModal;
 end;
end;

procedure TMusPaintForm.ToolSpeedButtonsClick(Sender: TObject);
begin
 Tool := TTool(TSpeedButton(Sender).Tag);
 Selected := False;
end;

procedure TMusPaintForm.BrushStyleSpeedButtonsClick(Sender: TObject);
begin
 BrushStyle := TBrushStyle(TSpeedButton(Sender).Tag);
end;

procedure TMusPaintForm.ColorBoxPanelResize(Sender: TObject);
begin
 ColorBoxPanel.Top := StatusBarPanel.Top - ColorBoxPanel.Height - 1;
end;

procedure TMusPaintForm.ColorPanelsClick(Sender: TObject);
begin
 FrontColorIndex := TPanel(Sender).Tag;
end;

procedure TMusPaintForm.BitmapSizePanelDblClick(Sender: TObject);
begin
 SetCanvasSizeCommandClick(Sender);
end;

procedure TMusPaintForm.NoteLengthPanelMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 P: TPoint;
begin
 P.X := X;
 P.Y := Y;
 P := NoteLengthPanel.ClientToScreen(P);
 NoteLengthPopupMenu.Popup(P.X, P.Y);
end;

procedure TMusPaintForm.NoteSystemPanelDblClick(Sender: TObject);
var
 ASystem: Integer;
begin
 ASystem := NoteSystem;
 ASystem := ASystem + 1;
 if NoteSystemsCount < ASystem then   ASystem := 0;
 NoteSystem := ASystem;
end;

procedure TMusPaintForm.TempoPanelDblClick(Sender: TObject);
begin
 Tempo := TempoSetDialog.Execute(Tempo);
end;

procedure TMusPaintForm.MagnificationPanelDblClick(Sender: TObject);
begin
 if Magnification = 1 then   Magnification := 2
 else                        Magnification := 1;
end;

procedure TMusPaintForm.ShowGridPanelDblClick(Sender: TObject);
begin
 ShowGrid := not ShowGrid;
end;

procedure TMusPaintForm.ToolPanelDblClick(Sender: TObject);
var
 ToolIndex: Integer;
begin
 ToolIndex := Ord(Tool);
 ToolIndex := ToolIndex + 1;
 if ToolsCount < ToolIndex then   ToolIndex := 0;
 Tool := TTool(ToolIndex);
end;

procedure TMusPaintForm.BrushStylePanelDblClick(Sender: TObject);
begin
 if bsClear = BrushStyle then   BrushStyle := bsSolid
 else                           BrushStyle := bsClear;
end;

procedure TMusPaintForm.SelColorPanelDblClick(Sender: TObject);
var
 AIndex: Integer;
begin
 AIndex := FrontColorIndex;
 AIndex := AIndex + 1;
 if ColorsCount < AIndex then   AIndex := 1;
 FrontColorIndex := AIndex;
end;

procedure TMusPaintForm.ShowASpeedButtonClick(Sender: TObject);
begin
 if ShowASpeedButtonDown then
  ShowNotASpeedButton.Down := True
 else
  ShowASpeedButton.Down := True;
 ShowASpeedButtonDown := not ShowASpeedButtonDown;
end;

procedure TMusPaintForm.HorizontalScrollBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
 if scPageUp = ScrollCode then
  ScrollPos := ScrollPos - PixelsPer64Note * 64 div CellWidth + 1;
 if scPageDown = ScrollCode then
  ScrollPos := ScrollPos + PixelsPer64Note * 64 div CellWidth - 1;
end;

procedure TMusPaintForm.HorizontalScrollBarChange(Sender: TObject);
begin
 Offset := HorizontalScrollBar.Position;
end;

procedure TMusPaintForm.MusicScrollBoxResize(Sender: TObject);
begin
 ArrangeImages;
end;

procedure TMusPaintForm.MusicPanelResize(Sender: TObject);
begin
 if not Refreshing then
  DrawMusic(NotesBitmap);
end;

procedure TMusPaintForm.MusicImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 P: TPoint;
begin
 P := MusicPointToBitmapPoint(Point(X, Y));
 if (P.Y < 0) or (BitmapHeight <= P.Y) or (BitmapWidth <= P.X) then
  begin
   Selected := False;
   Exit;
  end;
 MouseButtonDown := True;
 MouseLButtonDown := mbLeft = Button;
 MouseDownPoint := P;
 if MouseLButtonDown then
  SelectedColor := FrontColor
 else
  SelectedColor := BackColor;
 WorkBitmap.Assign(NotesBitmap);
 case Tool of
  tlPen: begin
          WorkBitmap.Canvas.Pixels[P.X, P.Y] := SelectedColor;
          DrawMusic(WorkBitmap);
         end;
  tlFill: ;
  tlLine: begin
           WorkBitmap.Canvas.Pixels[P.X, P.Y] := SelectedColor;
           DrawMusic(WorkBitmap);
          end;
  tlAngle: begin
            WorkBitmap.Canvas.Pixels[P.X, P.Y] := SelectedColor;
            DrawMusic(WorkBitmap);
           end;
  tlEllipse: begin
              WorkBitmap.Canvas.Pixels[P.X, P.Y] := SelectedColor;
              DrawMusic(WorkBitmap);
             end;
  tlSelect: if MouseLButtonDown then
             begin
              Selected := False;
              Selected := True;
              SelRect := Rect(P.X, P.Y, P.X + 1, P.Y + 1);
             end;
 end;
end;

procedure TMusPaintForm.MusicImageMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
 P: TPoint;
 R, R1, R2: TRect;
begin
 P := MusicPointToBitmapPoint(Point(X, Y));
 if MouseButtonDown or
     ((0 < P.X) and (P.X < BitmapWidth) and
       (0 < P.Y) and (P.Y < BitmapHeight)) then
  MousePointPanel.Caption := IntToStr(P.X) + ',' + IntToStr(P.Y)
 else
  MousePointPanel.Caption := '';
 if MouseButtonDown and (Tool in [tlLine, tlAngle, tlEllipse, tlSelect]) then
   SelSizePanel.Caption := + IntToStr(P.X - MouseDownPoint.X + 1) + 'x'
                            + IntToStr(P.Y - MouseDownPoint.Y + 1)
 else
  SelSizePanel.Caption := '';

 if MouseButtonDown then
  case Tool of
   tlPen: begin
           WorkBitmap.Canvas.Pixels[P.X, P.Y] := SelectedColor;
           DrawMusic(WorkBitmap);
          end;
   tlFill: ;
   tlLine: with WorkBitmap do
            begin
             Assign(NotesBitmap);
             with Canvas do
              begin
               Pen.Color := SelectedColor;
               Pen.Mode := pmCopy;
               Pen.Style := psSolid;
               Pen.Width := 1;
               Brush.Color := SelectedColor;
               Brush.Style := BrushStyle;
               MoveTo(MouseDownPoint.X, MouseDownPoint.Y);
               LineTo(P.X, P.Y);
               Pixels[P.X, P.Y] := SelectedColor;
              end;
             DrawMusic(WorkBitmap);
            end;
   tlAngle: with WorkBitmap do
             begin
              Assign(NotesBitmap);
              with Canvas do
               begin
                Pen.Color := SelectedColor;
                Pen.Mode := pmCopy;
                Pen.Style := psSolid;
                Pen.Width := 1;
                Brush.Color := SelectedColor;
                Brush.Style := BrushStyle;
               end;
              R := Rect(MouseDownPoint.X, MouseDownPoint.Y, P.X, P.Y);
              R := Rect(R.Left, R.Top, R.Right + 1, R.Bottom + 1);
              RectangleR(Canvas, R);
              DrawMusic(WorkBitmap);
             end;
   tlEllipse: with WorkBitmap do
               begin
                Assign(NotesBitmap);
                with Canvas do
                 begin
                  Pen.Color := SelectedColor;
                  Pen.Mode := pmCopy;
                  Pen.Style := psSolid;
                  Pen.Width := 1;
                  Brush.Color := SelectedColor;
                  Brush.Style := BrushStyle;
                 end;
                R := Rect(MouseDownPoint.X, MouseDownPoint.Y, P.X, P.Y);
                R := Rect(R.Left, R.Top, R.Right + 1, R.Bottom + 1);
                EllipseR(Canvas, R);
                DrawMusic(WorkBitmap);
               end;
   tlSelect: if MouseLButtonDown then
              begin
               R := Rect(MouseDownPoint.X, MouseDownPoint.Y, P.X, P.Y);
               R := Rect(R.Left, R.Top, R.Right + 1, R.Bottom + 1);
               if R.Top < 0 then   R.Top := 0;
               if BitmapHeight < R.Bottom then   R.Bottom := BitmapHeight;
               if R.Left < 0 then   R.Left := 0;
               if BitmapWidth < R.Right then   R.Right := BitmapWidth;
               SelRect := R;
               if X < 0 then
                with HorizontalScrollBar do
                 Position := Position - 1;
               if MusicImage.Width < X then
                with HorizontalScrollBar do
                 Position := Position + 1;
               if Y < 0 then
                with MusicScrollBox.VertScrollBar do
                 Position := Position - Increment;
               if MusicImage.Height < Y then
                with MusicScrollBox.VertScrollBar do
                 Position := Position + Increment;
              end;
  end;
end;

procedure TMusPaintForm.MusicImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 P: TPoint;
 R, R1, R2: TRect;
begin
 P := MusicPointToBitmapPoint(Point(X, Y));
 if MouseButtonDown then
  case Tool of
   tlPen: begin
           WorkBitmap.Canvas.Pixels[P.X, P.Y] := SelectedColor;
           UndoBitmap.Assign(NotesBitmap);
           UndoCommand.Enabled := True;
           NotesBitmap.Assign(WorkBitmap);
           Modified := True;
           DrawMusic(NotesBitmap);
          end;
   tlFill: with WorkBitmap do
            begin
             Assign(NotesBitmap);
             Canvas.Brush.Color := SelectedColor;
             Canvas.Brush.Style := bsSolid;
             Canvas.FloodFill(P.X, P.Y, Canvas.Pixels[P.X, P.Y], fsSurface);
             UndoBitmap.Assign(NotesBitmap);
             UndoCommand.Enabled := True;
             NotesBitmap.Assign(WorkBitmap);
             Self.Modified := True;
             DrawMusic(NotesBitmap);
            end;
   tlLine: begin
            UndoBitmap.Assign(NotesBitmap);
            UndoCommand.Enabled := True;
            NotesBitmap.Assign(WorkBitmap);
            Modified := True;
            DrawMusic(NotesBitmap);
           end;
   tlAngle: begin
             UndoBitmap.Assign(NotesBitmap);
             UndoCommand.Enabled := True;
             NotesBitmap.Assign(WorkBitmap);
             Modified := True;
             DrawMusic(NotesBitmap);
            end;
   tlEllipse: begin
               UndoBitmap.Assign(NotesBitmap);
               UndoCommand.Enabled := True;
               NotesBitmap.Assign(WorkBitmap);
               Modified := True;
               DrawMusic(NotesBitmap);
              end;
   tlSelect: if MouseLButtonDown then
              if bsClear = BrushStyle then
               with NotesBitmap.Canvas do
                begin
                 Brush.Color := BackColor;
                 Brush.Style := bsSolid;
                 FillRect(SelRect);
                 DrawMusic(NotesBitmap);
                end;
  end;
 MouseButtonDown := False;
 MouseLButtonDown := False;
end;

procedure TMusPaintForm.SelectImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 MouseButtonDown := True;
 MouseLButtonDown := mbLeft = Button;
 MouseDownPoint := Point(X, Y);
end;

procedure TMusPaintForm.SelectImageMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
 R: TRect;
begin
 if MouseLButtonDown then
  begin
   R := SelImagePos;
   R.Left := R.Left + (X - MouseDownPoint.X) div CellWidth  div Magnification;
   R.Top  := R.Top  + (Y - MouseDownPoint.Y) div CellHeight div Magnification;
   SelImagePos := R;
  end;
end;

procedure TMusPaintForm.SelectImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 MouseButtonDown := False;
 MouseLButtonDown := False;
end;

procedure TMusPaintForm.NCMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 Selected := False;
end;

procedure TMusPaintForm.NCMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
 MousePointPanel.Caption := '';
end;

function TMusPaintForm.GetBackColor: TColor;
begin
 Result := MuseColors[BackColorIndex];
end;

function TMusPaintForm.GetFrontColor: TColor;
begin
 Result := MuseColors[FrontColorIndex];
end;

procedure TMusPaintForm.SetBitmapFileName(AFileName: TFileName);
begin
 if not Refreshing then
  if FBitmapFileName = AFileName then   Exit;
 FBitmapFileName := AFileName;
 Caption := 'MusPaint - ' + ExtractFileName(AFileName);
 if FileExists(AFileName) then
  begin
   SaveBitmapDialog.FileName := ExtractFileName(AFileName);
   SaveBitmapDialog.InitialDir := ExtractFilePath(AFileName);
   MusFileName := ChangeFileExt(AFileName, '.mus');
  end;
end;

procedure TMusPaintForm.SetBitmapWidth(AWidth: Integer);
begin
 if not Refreshing then
  if FBitmapWidth = AWidth then   Exit;
 if not Refreshing then
  Selected := False;
 FBitmapWidth := AWidth;
 NotesBitmap.Width := AWidth;
 Modified := True;
 CanvasSizeDialog.BitmapWidth := AWidth;
 BitmapSizePanel.Caption := IntToStr(BitmapWidth) + 'x'
                             + IntToStr(BitmapHeight);
 if not Refreshing then
  begin
   DrawMusic(NotesBitmap);
  end;
end;

procedure TMusPaintForm.SetBrushStyle(ABrushStyle: TBrushStyle);
begin
 if not Refreshing then
  if FBrushStyle = ABrushStyle then   Exit;
 FBrushStyle := ABrushStyle;
 if bsClear = ABrushStyle then
  begin
   bsClearSpeedButton.Down := True;
   bsClearCommand.Checked := True;
   bsSolidCommand.Checked := False;
   BrushStylePanel.Caption := 'NA';
  end
 else if bsSolid = ABrushStyle then
  begin
   bsSolidSpeedButton.Down := True;
   bsClearCommand.Checked := False;
   bsSolidCommand.Checked := True;
   BrushStylePanel.Caption := 'tB';
  end;
end;

procedure TMusPaintForm.SetCellWidth(AWidth: Integer);
begin
 if not Refreshing then
  if FCellWidth = AWidth then   Exit;
 FCellWidth := AWidth;
 if not Refreshing then
  begin
   DrawMusic(NotesBitmap);
   SelImagePos := SelImagePos;
  end;
end;

procedure TMusPaintForm.SetCheckSave(ACheck: Boolean);
begin
 FCheckSave := ACheck;
 CheckSaveCommand.Checked := ACheck;
end;

procedure TMusPaintForm.SetFrontColorIndex(AIndex: Integer);
var
 i: Integer;
begin
 if not Refreshing then
  if FFrontColorIndex = AIndex then   Exit;
 FFrontColorIndex := AIndex;
 for i := 1 to ColorsCount do
  begin
   ColorCommands[i].Checked := False;
   ColorPCommands[i].Checked := False;
  end;
 ColorCommands[AIndex].Checked := True;
 ColorPCommands[AIndex].Checked := True;
 FrontColorPanel.Color := MuseColors[AIndex];
 SelColorPanel.Color := MuseColors[AIndex];
 SelColorPanel.Caption := MuseColorNames[AIndex];
end;

procedure TMusPaintForm.SetMagnification(AMagnification: Integer);
begin
 if not Refreshing then
  if FMagnification = AMagnification then   Exit;
 FMagnification := AMagnification;
 if 1 = AMagnification then
  begin
   ZoomUpCommand.Enabled := True;
   ZoomNormalCommand.Enabled := False;
  end
 else
  begin
   ZoomUpCommand.Enabled := False;
   ZoomNormalCommand.Enabled := True;
  end;
 MagnificationPanel.Caption := IntToStr(AMagnification * 100) + '%';
 MusicScrollBox.VertScrollBar.Increment := CellHeight * AMagnification;
 if not Refreshing then
  begin
   ArrangeImages;
   DrawMusic(NotesBitmap);
   SelImagePos := SelImagePos;
  end;
end;

procedure TMusPaintForm.SetModified(AModified: Boolean);
begin
 if not Refreshing then
  if FModified = AModified then   Exit;
 FModified := AModified;
 UndoCommand.Enabled := FModified;
end;

procedure TMusPaintForm.SetMusFileName(AFileName: TFileName);
begin
 if not Refreshing then
  if FMusFileName = AFileName then   Exit;
 FMusFileName := AFileName;
 SaveMusDialog.FileName := ExtractFileName(AFileName);
 SaveMusDialog.InitialDir := ExtractFilePath(AFileName);
end;

procedure TMusPaintForm.SetNoteLength(ALength: Integer);
var
 i: Integer;
begin
 if not Refreshing then
  if FNoteLength = ALength then   Exit;
 FNoteLength := ALength;
 NoteLengthDialog.NoteLength := ALength;
 for i := 0 to NoteLengthsCount do
  begin
   NoteCommands[i].Checked := False;
   NotePCommands[i].Checked := False;
  end;
 NoteCommands[NoteLengthDialog.NoteLengthIndex].Checked := True;
 NotePCommands[NoteLengthDialog.NoteLengthIndex].Checked := True;
 NoteLengthPanel.Caption := NoteLengthDialog.NoteLengthString;
 CellWidth := ALength * PixelsPer64Note;
 DrawSelectImage;
 SelImagePos := SelImagePos;
end;

procedure TMusPaintForm.SetNoteSystem(ASystem: Integer);
var
 i: Integer;
begin
 if not Refreshing then
  if FNoteSystem = ASystem then   Exit;
 FNoteSystem := ASystem;
 for i := 0 to NoteSystemsCount do
  NoteSystemCommands[i].Checked := False;
 NoteSystemCommands[ASystem].Checked := True;
 NoteSystemPanel.Caption := NoteSystemNames[ASystem];
end;

procedure TMusPaintForm.SetOffset(AOffset: Integer);
begin
 if not Refreshing then
  if FOffset = AOffset then   Exit;
 FOffset := AOffset;
 HorizontalScrollBar.Position := AOffset;
 if not Refreshing then
  begin
   DrawMusic(NotesBitmap);
   SelImagePos := SelImagePos;
  end;
end;

procedure TMusPaintForm.SetSelected(ASelected: Boolean);
begin
 if not Refreshing then
  if FSelected = ASelected then   Exit;
 FSelected := ASelected;
 SelectImage.Visible := ASelected;
 DeleteCommand.Enabled := ASelected;
 if ASelected then
  DrawSelectImage
 else
  begin
   SelectImage.Left := -SelectImage.Width;
   UndoBitmap.Assign(NotesBitmap);
   UndoCommand.Enabled := True;
   NotesBitmap.Canvas.Draw(SelImagePos.Left, SelImagePos.Top, SelectBitmap);
   Modified := True;
   DrawMusic(NotesBitmap);
   with SelectImage do   Left := -Width;
  end;
end;

procedure TMusPaintForm.SetSelImagePos(APos: TRect);
var
 H: Integer;
 L: Longint;
 T: Integer;
 W: Integer;
begin
 FSelImagePos := APos;
 if Selected then
  with SelectImage do
   begin
    H := Picture.Bitmap.Height * Magnification;
    L := Longint(APos.Left - Offset) * CellWidth * Magnification;
    T := (TopBlanc + APos.Top * CellHeight) * Magnification;
    W := Picture.Bitmap.Width * Magnification;
    if (0 <= L + W) and (L < MusicImage.Width) then
     SetBounds(L, T, W, H)
    else
     Left := -Width;
   end;
end;

procedure TMusPaintForm.SetSelRect(ARect: TRect);
var
 x: Longint;
begin
 if not Refreshing then
  if (FSelRect.Left = ARect.Left) and (FSelRect.Top = ARect.Top) and
      (FSelRect.Right = ARect.Right) and (FSelRect.Bottom = ARect.Bottom) then
   Exit;
 FSelRect := ARect;
 if not Refreshing and Selected then
  with SelectBitmap do
   begin
    Height := SelRect.Bottom - SelRect.Top;
    Width  := SelRect.Right  - SelRect.Left;
    Canvas.CopyRect(Rect(0, 0, Width, Height), NotesBitmap.Canvas, SelRect);
    SelImagePos := ARect;
    DrawSelectImage;
   end;
end;

procedure TMusPaintForm.SetShowGrid(AGrid: Boolean);
begin
 if not Refreshing then
  if FShowGrid = AGrid then   Exit;
 FShowGrid := AGrid;
 ShowGridCommand.Checked := AGrid;
 if AGrid then
  ShowGridPanel.Caption := 'on'
 else
  ShowGridPanel.Caption := 'off';
 DrawMusic(NotesBitmap);
end;

procedure TMusPaintForm.SetTempo(ATempo: Integer);
begin
 FTempo := ATempo;
 TempoPanel.Caption := '%' + IntToStr(Tempo);
end;

procedure TMusPaintForm.SetTool(ATool: TTool);
var
 i: Integer;
begin
 if not Refreshing then
  if FTool = ATool then   Exit;
 FTool := ATool;
 for i := 0 to ToolsCount do
  ToolCommands[i].Checked := False;
 ToolCommands[Ord(ATool)].Checked := True;
 ToolSpeedButtons[Ord(ATool)].Down := True;
 Selected := False;
 ToolPanel.Caption := ToolNames[Ord(ATool)];
 BrushStyleBasePanel.Visible := ATool in [tlAngle, tlEllipse, tlSelect];
end;

procedure TMusPaintForm.WMDropFiles(var Message: TWMDropFiles);
var
 FileNamePChar: array[0..79] of Char;
 FileName: String;
 TempBitmap: TBitmap;
begin
 Application.Minimize;
 Application.Restore;
 {SetForegroundWindow(Handle);}
 Selected := False;
 TempBitmap := TBitmap.Create;
 DragQueryFile(Message.Drop, 0, FileNamePChar, 80);
 FileName := StrPas(FileNamePChar);
 DragFinish(Message.Drop);
 if FileExists(FileName) then
  try
   TempBitmap.LoadFromFile(FileName);
   TempBitmap.Width := 0;
   LoadBitmapWizard.FileName := '';
   LoadBitmap(FileName);
  except
   on EInvalidGraphic do
    begin
     TempBitmap.Free;
     Exit;
    end;
  end;
 TempBitmap.Free;
end;

procedure TMusPaintForm.ArrangeImages;
var
 w: Integer;
begin
 w := MusicLWidth * Magnification;
 MusicLImage.Height := MusicHeight * Magnification;
 MusicLImage.Width  := w;
 MusicPanel.Height := MusicLImage.Height;
 MusicPanel.Left   := w;
 MusicPanel.Width  := MusicScrollBox.Width - w;
 HorizontalScrollBar.Left := w;
 HorizontalScrollBar.Width := MusicScrollBox.Width - w;
 w := w div 3;
 ShowGSpeedButton.Width := w;
 ShowTSpeedButton.Left := w;
 ShowTSpeedButton.Width := w;
 ShowASpeedButton.Left := w * 2;
 ShowASpeedButton.Width := w;
end;

procedure TMusPaintForm.DrawMusic(var NotesBitmap: TBitmap);
var
 i, j: Integer;
 x: Longint;
 y: Integer;
 C: TColor;
 S: String;
begin
 if CellWidth < EndBlanc then   i := EndBlanc
 else                           i := CellWidth;
 x := ((Longint(BitmapWidth) * CellWidth + i) * Magnification - MusicImage.Width)
       div CellWidth div Magnification;
 if x < 0 then   x := 0;
 with HorizontalScrollBar do
  begin
   Max := x;
   Enabled := not(0 = x);
   if x < Offset then
    begin
     Refreshing := True;
     Offset := x;
     Refreshing := False;
    end;
  end;

 with MusicBitmap do
  begin
   Height := TopBlanc + BitmapHeight * CellHeight + 1 + BottomBlanc;
   i := MusicImage.Width div Magnification;
   if 0 < MusicImage.Width mod Magnification then   i := i + 1;
   Width := i;
   with Canvas do
    begin
     Brush.Color := BackColor;
     Brush.Style := bsSolid;
     FillRect(Rect(0, 0, Width, Height));
     Pen.Mode := pmCopy;
     Pen.Style := psSolid;
     Pen.Width := 1;
     i := -1;
     while True do
      begin
       i := i + 1;
       if Longint(i) * LineSpace < Longint(CellWidth) * (Offset - 1) then
        Continue;
       if Longint(CellWidth) * Offset + Width < Longint(i) * LineSpace then
        Break;
       x := Longint(LineSpace) * i - Longint(CellWidth) * Offset;
       if 0 = i mod 4 then
        begin
         S := IntToStr(i div 4 + 1);
         for j := 1 to Length(S) do
          CopyRect(Rect(x + j * 7 - 3, 319, x + j * 7 + 3, 329),
                    FigureBitmap.Canvas,
                     Rect((Integer(S[j]) - $30{'0'} ) * 6, 0,
                           (Integer(S[j]) - $30{'0'} ) * 6 + 6, 10));
         Pen.Color := MusicLineColor;
        end
       else
        Pen.Color := BeatLineColor;
       MoveTo(x,      0);
       LineTo(x, Height);
      end;
     for i := 10 to 20 do
      if i in [10, 11, 12, 13, 14, 16, 17, 18, 19, 20] then
       begin
        Pen.Color := MusicLineColor;
        y := TopBlanc + 2 + 10 * i;
        MoveTo(    0, y);
        LineTo(Width, y);
        MoveTo(    0, y + 1);
        LineTo(Width, y + 1);
       end;
     for i := 0 to 27 do
      if not(i in [10, 11, 12, 13, 14, 16, 17, 18, 19, 20]) then
       begin
        Pen.Color := AdditionalLineColor;
        x := Longint(BitmapWidth - Offset) * CellWidth;
        if Width < x then   x := Width;
        y := TopBlanc + 2 + 10 * i;
        MoveTo(0, y);
        LineTo(x, y);
        MoveTo(0, y + 1);
        LineTo(x, y + 1);
       end;
     x := Longint(BitmapWidth - Offset) * CellWidth;
     if x < Width then
      begin
       Pen.Color := MusicLineColor;
       for i := 0 to 7 do
        if i in [0, 3, 4, 5, 6, 7] then
         begin
          MoveTo(x + i, 127);{TopBlanc + 2 + 10 * 10}
          LineTo(x + i, 167);{TopBlanc + 2 + 10 * 14}
          MoveTo(x + i, 187);{TopBlanc + 2 + 10 * 16}
          LineTo(x + i, 227);{TopBlanc + 2 + 10 * 20}
         end;
      end;
     if ShowGrid then
      begin
       Pen.Color := GridColor;
       for i := 0 to 56 do
        begin
         x := Longint(BitmapWidth - Offset) * CellWidth;
         if Width < x then   x := Width;
         y := TopBlanc + CellHeight * i;
         MoveTo(0, y);
         LineTo(x, y);
        end;
       i := 0;
       while True do
        begin
         if Width < Longint(CellWidth * i) then   Break;
         if BitmapWidth - Offset < i then   Break;
         x := CellWidth * i;
         MoveTo(x, TopBlanc);
         LineTo(x, Height - BottomBlanc);
         i := i + 1;
        end;
      end;
     for x := Offset to NotesBitmap.Width - 1 do
      if Width < Longint(x - Offset) * CellWidth then   Break
      else
       for y := 0 to NotesBitmap.Height - 1 do
        begin
         C := NotesBitmap.Canvas.Pixels[x, y];
         if BackColor = C then   Continue;
         Brush.Color := C;
         Pen.Color   := BorderColor;
         Rectangle((x - Offset) * CellWidth,
                    TopBlanc + y * CellHeight,
                     (x - Offset + 1) * CellWidth + 1,
                      TopBlanc + (y + 1) * CellHeight + 1);
        end;
    end;
  end;
 with MusicImage.Picture.Bitmap do
  begin
   Width  := MusicBitmap.Width  * Magnification;
   Height := MusicBitmap.Height * Magnification;
   Canvas.StretchDraw(Rect(0, 0, Width, Height), MusicBitmap);
  end;
end;

procedure TMusPaintForm.DrawSelectImage;
var
 x, y: Integer;
 C: TColor;
begin
 with SelectImage.Picture.Bitmap do
  begin
   Height := SelectBitmap.Height * CellHeight + 1;
   Width  := SelectBitmap.Width  * CellWidth + 1;
   with Canvas do
    begin
     Brush.Color := BackColor;
     Brush.Style := bsSolid;
     Pen.Color := BorderColor;
     Pen.Mode  := pmCopy;
     Pen.Style := psSolid;
     Pen.Width := 1;
     FillRect(Rect(0, 0, Width - 1, Height - 1));
     for x := 0 to SelectBitmap.Width - 1 do
      for y := 0 to SelectBitmap.Height - 1 do
       begin
        C := SelectBitmap.Canvas.Pixels[x, y];
        if BackColor = C then   Continue;
        Brush.Color := C;
        Pen.Color   := BorderColor;
        Rectangle(x * CellWidth, y * CellHeight,
                   (x + 1) * CellWidth + 1, (y + 1) * CellHeight + 1);
       end;
     for y := 0 to Height - 1 do
      begin
       if y mod 4 < 2 then C := BackColor
       else                C := BorderColor;
       Pixels[0, y]                      := C;
       Pixels[Width - 1, Height - 1 - y] := C;
      end;
     for x := 0 to Width - 1 do
      begin
       if x mod 4 < 2 then C := BackColor
       else                C := BorderColor;
       Pixels[x, 0]                      := C;
       Pixels[Width - 1 - x, Height - 1] := C;
      end;
    end;
  end;
 SelImagePos := SelImagePos;
end;

procedure TMusPaintForm.LoadBitmap(AFileName: TFileName);
var
 MR: Integer;
begin
 Selected := False;
 if Modified and CheckSave then
  begin
   MessageBeep(MB_ICONQUESTION);
   MR := MessageBox(Handle, 'rbg}bv̕ύXۑ܂H',
                     'MusPaint', MB_YESNOCANCEL);
   if IDYES = MR then
    if SaveBitmap then
    else
     Exit
   else if IDNO = MR then
   else if IDCANCEL = MR then
    Exit;
  end;
 if LoadBitmapWizard.Execute(AFileName) then
  begin
   NotesBitmap.Assign(LoadBitmapWizard.NotesBitmap);
   BitmapWidth := NotesBitmap.Width;
   Modified := LoadBitmapWizard.Modified;
   BitmapFileName := LoadBitmapWizard.FileName;
   DrawMusic(NotesBitmap);
  end;
end;

procedure TMusPaintForm.NewBitmap;
var
 MR: Integer;
begin
 Selected := False;
 if Modified and CheckSave then
  begin
   MessageBeep(MB_ICONQUESTION);
   MR := MessageBox(Handle, 'rbg}bv̕ύXۑ܂H',
                     'MusPaint', MB_YESNOCANCEL);
   if IDYES = MR then
    if SaveBitmap then
    else
     Exit
   else if IDNO = MR then
   else if IDCANCEL = MR then
    Exit;
  end;
 with NotesBitmap do
  begin
   Height := BitmapHeight;
   Width  := BitmapWidth;
   with Canvas do
    begin
     Brush.Color := BackColor;
     Brush.Style := bsSolid;
     FillRect(Rect(0, 0, Width, Height));
    end;
  end;
 DrawMusic(NotesBitmap);
 Modified := False;
 BitmapFileName := '()';
end;

function TMusPaintForm.MusicPointToBitmapPoint(APoint: TPoint): TPoint;
begin
 Result.X := APoint.X div CellWidth div Magnification + Offset;
 Result.Y := ((APoint.Y div Magnification) - TopBlanc) div CellHeight;
end;

function TMusPaintForm.MusicRectToBitmapRect(ARect: TRect): TRect;
begin
 Result.TopLeft     := MusicPointToBitmapPoint(ARect.TopLeft);
 Result.BottomRight := MusicPointToBitmapPoint(ARect.BottomRight);
end;

function TMusPaintForm.BitmapPointToMusicPoint(APoint: TPoint): TPoint;
begin
 Result.X := Longint(APoint.X - Offset) * CellWidth * Magnification;
 Result.Y := (APoint.Y * CellHeight + TopBlanc) * Magnification;
end;

function TMusPaintForm.BitmapRectToMusicRect(ARect: TRect): TRect;
begin
 Result.TopLeft     := BitmapPointToMusicPoint(ARect.TopLeft);
 Result.BottomRight := BitmapPointToMusicPoint(ARect.BottomRight);
end;

function TMusPaintForm.SaveBitmap: Boolean;
begin
 Selected := False;
 Result := True;
 if FileExists(BitmapFileName) then
  begin
   NotesBitmap.SaveToFile(BitmapFileName);
   Modified := False;
  end
 else
  Result := SaveBitmapAs;
end;

function TMusPaintForm.SaveBitmapAs: Boolean;
begin
 Selected := False;
 Result := False;
 with SaveBitmapDialog do
  if Execute then
   begin
    BitmapFileName := FileName;
    NotesBitmap.SaveToFile(BitmapFileName);
    Result := True;
    Modified := False;
    InitialDir := ExtractFilePath(FileName);
    FileName  := ExtractFileName(FileName);
   end;
end;

procedure TMusPaintForm.SaveMus;
begin
 Selected := False;
 if FileExists(MusFileName) then
  SaveMusFile
 else
  SaveMusAs;
end;

procedure TMusPaintForm.SaveMusAs;
begin
 Selected := False;
 with SaveMusDialog do
  if Execute then
   begin
    MusFileName := FileName;
    SaveMusFile;
   end;
end;

procedure TMusPaintForm.SaveMusFile;
var
 F: TextFile;
 i, x, y: Integer;
 Note, Notes, LastNote, LastNotes: String;
 NoteLengthSet: Boolean;
 MuseColorUsed: array[1..ColorsCount] of Boolean;
 OutStr: String;
begin
 for i := 1 to ColorsCount do
  MuseColorUsed[i] := False;
 for i := 1 to ColorsCount do
  for x := 0 to NotesBitmap.Width - 1 do
   begin
    for y := 0 to NotesBitmap.Height - 1 do
     begin
      if MuseColors[i] = NotesBitmap.Canvas.Pixels[x, y] and $FFFFFF then
       begin
        MuseColorUsed[i] := True;
        Break;
       end;
     end;
    if MuseColorUsed[i] then   Break;
   end;

 AssignFile(F, MusFileName);
 Rewrite(F);
 WriteLn(F, '*MARK""');
 WriteLn(F, '\=%%', IntToStr(Tempo));
 with NotesBitmap do
  for i := 1 to ColorsCount do
   if MuseColorUsed[i] then
    begin
     OutStr := '@' + MuseColorNames[i] + 'T0#' + MuseColorNames[i] + '0' +
                'x' + IntToStr(NoteSystem) + '?0v0';
     LastNotes := '';
     NoteLengthSet := False;
     for x := 0 to Width - 1 do
      begin
       LastNote := '';
       Notes := '';
       for y := 0 to Height - 1 do
        if Canvas.Pixels[x, Height - y - 1] and $FFFFFF = MuseColors[i] then
         begin
          Note := NoteNodes[NoteSystem][y + 1];
          if (3 = Length(LastNote)) and (LastNote[2] = Note[2]) then
           begin
            LastNote := Note;
            Note[1] := Note[3];
            Note[0] := Chr(1);
           end
          else
           LastNote := Note;
          Notes := Notes + Note;
         end;
       if ('' <> Notes) and (0 = CompareStr(LastNotes, Notes)) then
        begin
         LastNotes := Notes;
         Notes := ',';
        end
       else
        begin
         LastNotes := Notes;
         if Notes = '' then
          Notes := '_'
         else
          Notes := '['+Notes+']';
        end;
       if not NoteLengthSet then
        begin
         Notes := Notes + '64`' + IntToStr(NoteLength);
         NoteLengthSet := True;
        end;
       OutStr := OutStr + Notes;
       if 80 <= Length(OutStr) then
        begin
         WriteLn(F, Copy(OutStr, 1, 80));
         OutStr := '  ' + Copy(OutStr, 81, 175);
        end;
      end;
     OutStr := OutStr + 'v100o4';
     if Length(OutStr) <= 80 then
      WriteLn(F, OutStr)
     else
      begin
       WriteLn(F, Copy(OutStr,  1,  80));
       WriteLn(F, '  ' + Copy(OutStr, 81, 175));
      end;
    end;
 WriteLn(F, '%');
 WriteLn(F, '*MARK""');
 CloseFile(F);
end;

procedure TMusPaintForm.ShowKeyboardCommandClick(Sender: TObject);
begin
 KeyboardForm.Show;
end;

end.
